#include "Ball.h" 
#include "MoveableObject.h" 
#include "PhysicsObject.h" 
#include "Sound.h"
#include "TennisPlayer.h"

Ball::Ball(PhysicsObject *n_physicsObject, osg::MatrixTransform *n_graphicsObject) : MoveableObject(n_physicsObject, n_graphicsObject)
{
	
	bounce = .99;
	friction = .02;
	bounce_count = 0;
	lastHit = 0;
	
	bouncingSound = new Sound("../samples/bounce_ball.wav");
	bouncingSound->setAmbient(false);
	
	//printf("Created Ball.\n");
	
}

Ball::~Ball()
{
	//printf("Deleted Ball.\n");
	
}

void 
Ball::collision(MoveableObject *mo, dContact thisContact)
{
	//printf("I ball collision.\n");
	
	dGeomID own_object = getPhysicsObject();
	dGeomID other_object = mo->getPhysicsObject();
	
	dJointID collisionJoint;
	
	//dContact thisContact;
		
	thisContact.surface.mode = dContactBounce;
	thisContact.surface.mu = friction * mo->getFriction();
	//thisContact.surface.mu2 = 1000;
	thisContact.surface.bounce = bounce * mo->getBounce();
	thisContact.surface.bounce_vel = 0;	
	
	collisionJoint = dJointCreateContact(physicsObject->getWorld(), physicsObject->getGroup(), &thisContact);
		
	dJointAttach(collisionJoint, dGeomGetBody(own_object), dGeomGetBody(other_object));
	
	bounceSound();
}

void 
Ball::setLastHit(WorldObject *hitObject)
{
	if(hitObject->asTennisPlayer() != NULL)
	{
		lastHit = hitObject;
		//Mste man inte nollstlla denna, annars kommer bounce_count>2 checken
		//nedanfr inte att funka korrekt ????
		bounce_count = 0;
	}
	else
	{
		bounce_count++;
		if(bounce_count >= 2)
		{
			bounce_count = 0;
			lastHit = hitObject;
		}
	}
}

WorldObject * 
Ball::getLastHit()
{
	return lastHit;
}

void Ball::bounceSound()
{
	const dReal *linearVel = dBodyGetLinearVel(dGeomGetBody(getPhysicsObject()));
		
	osg::Vec3 linearVelVec;
		
	linearVelVec._v[0] = linearVel[0];
	linearVelVec._v[1] = linearVel[1];
	linearVelVec._v[2] = linearVel[2];
		
	if(linearVelVec.length() > 3)
	{
		//std::cout << "Bollens fart r hgre n 3, det blir ljud!" << std::endl;
		bouncingSound->playAtPos(this, listener);
		bouncingSound->play();
	}
}
