#include "MortalTennis.h"
#include "ScoreBoard.h"
#include "Sound.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

using namespace openalpp;



MortalTennis::MortalTennis()
{
	rakClientInterface = 0;
	rakServerInterface = 0;
	scoreBoard = new ScoreBoard();
	world.setScoreBoard(scoreBoard);
	playerView = EYEVIEW;
	clientViewAngle = 0.0;
	batmanTextCounter = 0;
	//topRightCameraType = NOCAMERA;
	musicOn = false;
}
	
void MortalTennis::createScoreBoard()
{
	rootNode->addChild(scoreBoard->getGeode());
}

void MortalTennis::initBatmanText()
{
	osg::Geode* HUDGeode = new osg::Geode();
	batmanText = new osgText::Text();
	powerText = new osgText::Text();
		
	osg::Projection* HUDProjectionMatrix = new osg::Projection;
	HUDProjectionMatrix->setMatrix(osg::Matrix::ortho2D(0,640,0,480));
	
	osg::MatrixTransform* HUDModelViewMatrix = new osg::MatrixTransform;
	HUDModelViewMatrix->setMatrix(osg::Matrix::identity());
	
	HUDModelViewMatrix->setReferenceFrame(osg::Transform::ABSOLUTE_RF);
	
	rootNode->addChild(HUDProjectionMatrix);
	HUDProjectionMatrix->addChild(HUDModelViewMatrix);
	
	HUDModelViewMatrix->addChild(HUDGeode);
	
	HUDGeode->addDrawable(batmanText);
	HUDGeode->addDrawable(powerText);
	
	batmanText->setCharacterSize(100);
	batmanText->setFont("../fonts/mickey.ttf");
	batmanText->setText("");
	batmanText->setAxisAlignment(osgText::Text::SCREEN);
	batmanText->setPosition( osg::Vec3(200,180,0) );
	batmanText->setColor( osg::Vec4(0.9f, 0.2f, 0.2f, 1.0f) );
	
	powerText->setCharacterSize(100);
	powerText->setFont("../fonts/mickey.ttf");
	powerText->setText("");
	powerText->setAxisAlignment(osgText::Text::SCREEN);
	powerText->setPosition( osg::Vec3(200,180,0) );
	powerText->setColor( osg::Vec4(0.9f, 0.2f, 0.2f, 1.0f) );
		
	osg::StateSet* HUDStateSet = new osg::StateSet();
	HUDGeode->setStateSet(HUDStateSet);
	HUDStateSet->setMode(GL_DEPTH_TEST,osg::StateAttribute::OFF);

}

osgText::Text* MortalTennis::getBatmanText()
{
	return batmanText;
}

osgText::Text* MortalTennis::getPowerText()
{
	return powerText;
}

Producer::CameraConfig* MortalTennis::setupCamera()
{
	Producer::CameraConfig* myCameraConfig = new Producer::CameraConfig();
	Producer::Camera* mainCamera = new Producer::Camera();
	Producer::Camera* topRightCamera = new Producer::Camera();
	statusBarCamera = new Producer::Camera();
	
	
	mainCamera->setProjectionRectangle(0.0f, 1.0f, 0.0f, 1.0f);
	mainCamera->setShareLens(false);
	mainCamera->getLens()->setAutoAspect(false);
	mainCamera->getLens()->setAspectRatio(1.7);
	
	//std::cout<<"mainview HFov: "<<mainCamera->getLens()->getHorizontalFov()<<std::endl;
	//std::cout<<"mainview VFov: "<<mainCamera->getLens()->getVerticalFov()<<std::endl;
	
	topRightCamera->setProjectionRectangle(0.9999f, 1.0f, 0.9999f, 1.0f);
	topRightCamera->setShareLens(false);
	topRightCamera->getLens()->setAutoAspect(true);
	
	statusBarCamera->setProjectionRectangle(0.0f, 0.02f, 0.0f, 0.18f);
	statusBarCamera->setShareLens(false);
	statusBarCamera->getLens()->setAutoAspect(true);
	
	
	myCameraConfig->addCamera("mainCamera",mainCamera);
	myCameraConfig->addCamera("topRightCamera",topRightCamera);
	myCameraConfig->addCamera("statusBarCamera",statusBarCamera);
	
	
	renderSurface = mainCamera->getRenderSurface();
	renderSurface->useCursor(false);
	renderSurface->useBorder(true);
	renderSurface->setWindowRectangle(500,150,640,480); //mste ndras i IOHandler.cpp ox...
	renderSurface->fullScreen(false);
	renderSurface->setWindowName("Mortal Tennis");

	topRightCamera->setRenderSurface(renderSurface);
	statusBarCamera->setRenderSurface(renderSurface);

	return myCameraConfig;
}

void MortalTennis::initNetwork(char *serverName, int type)
{
	switch(type)
	{
		case SERVER:
			
			rakClientInterface = 0;
			sn = new ServerNetwork(&world, scoreBoard);
			if(!sn->init())
				viewer->setDone(true); //stng ner ifall servern inte kunde starta
			rakServerInterface = sn->getInterface();	
			
			std::cout<<"Jag r en server"<<std::endl;
			
				
		break;
		
		case CLIENT:
			
			rakServerInterface = 0;
			cn = new ClientNetwork(&world, viewer, mc, scoreBoard);
			if(!cn->init(serverName)) //stng ner ifall klienten inte kunde starta
				viewer->setDone(true);
			rakClientInterface = cn->getInterface();
			
			std::cout<<"Jag r en klient"<<std::endl;
			
		break;
		
		default:
		break;
	
	}
}

void MortalTennis::loadModel()
{
	loadedModel = osgDB::readNodeFile("../models/MortalTennisWorld.osg");	

	if (!loadedModel) 
	{
		std::cout<<"Couldn't load world"<<std::endl;
		return;
	}
	
	rootNode->addChild(loadedModel.get());
	
	//Kommentera bort de hr tre raderna om visitorn inte ska kras
	CreateWorldObjectsVisitor *createWorldObjectsVisitor;
	createWorldObjectsVisitor = new CreateWorldObjectsVisitor(&world);
	loadedModel->accept(*createWorldObjectsVisitor);
	
	world.moveableObjects = createWorldObjectsVisitor->getMoveableObjects();
	world.numMoveableObjects = createWorldObjectsVisitor->getNumMoveableObjects();
	
	world.worldObjects = createWorldObjectsVisitor->getWorldObjects();
	world.numWorldObjects = createWorldObjectsVisitor->getNumWorldObjects();
	
	delete createWorldObjectsVisitor;
}

void MortalTennis::setupViewer()
{
	//la till s att man kan konfigurerar kameran, ta bort muspekaren osv..
	Producer::CameraConfig* myCameraConfig = setupCamera();
	
	viewer = new osgProducer::Viewer(myCameraConfig);
	//viewer->setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
	//viewer->setUpViewer(osgProducer::Viewer::NO_EVENT_HANDLERS);
	viewer->setUpViewer(osgProducer::Viewer::HEAD_LIGHT_SOURCE |
						osgProducer::Viewer::SKY_LIGHT_SOURCE);
	//viewer->setUpViewer(osgProducer::Viewer::SKY_LIGHT_SOURCE);
	
	viewer->setSceneData(rootNode);
	viewer->realize();
	
	
	// set all the sceneview's up so that their left and right add cull masks are set up.
    for(osgProducer::OsgCameraGroup::SceneHandlerList::iterator itr=viewer->getSceneHandlerList().begin();
        itr!=viewer->getSceneHandlerList().end();
        ++itr)
    {
        // switch off small feature culling to prevent the light points from being culled.
        osgUtil::SceneView* sceneview = (*itr)->getSceneView();
        sceneview->setCullingMode( sceneview->getCullingMode() & ~osg::CullStack::SMALL_FEATURE_CULLING);
    }

	
		
	mc = new MoveCameraManipulator(rakClientInterface, this);
	mc->setNode(loadedModel.get());
	if(rakClientInterface != 0)
		mc->setCameraManually(true);
	viewer->addCameraManipulator(mc);
	
	KeyEventHandler *key = new KeyEventHandler(rakClientInterface, viewer, renderSurface, this);
	viewer->getEventHandlerList().push_front(key); 

}


void MortalTennis::setPlayerView(int view)
{
	playerView = view;
}

void MortalTennis::setTopRightCameraType(int type)
{
	topRightCameraType = type;
}

void MortalTennis::setStatusBarCamera()
{
	viewer->getCameraConfig()->
		findCamera("statusBarCamera")->setViewByMatrix(
			Producer::Matrix::translate(50+world.getPlayerNr(),51.05,-5) *
			Producer::Matrix::rotate(M_PI,0,0,1));
}

void MortalTennis::applyTopRightCamera(int type)
{
	//hur str rektangel uppe i hgra hrnet ska vara..
	//left, right, bottom, top
	osg::Vec4 projRec(0.67f, 1.0f, 0.70f, 1.0f);
	
	if(type == SCOREBOARD)
	{
		//stt scoreBoard kamerans position...
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->setViewByMatrix(
				Producer::Matrix::translate(-181,23,-385));
		
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->
				setProjectionRectangle(projRec.x(),projRec.y(),projRec.z(),projRec.w());
	}
	else if (type == OVERVIEW_A)
	{
		//stt kameran s man har versikt ver planen...
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->setViewByMatrix(
				Producer::Matrix::translate(12,23,-15) *
				Producer::Matrix::rotate(M_PI/7,0,0,1) *
				Producer::Matrix::rotate(-M_PI/3,1,0,0) );
								
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->
				setProjectionRectangle(projRec.x(),projRec.y(),projRec.z(),projRec.w());
	
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->getLens()->setAspectRatio(2.0);
		
	}
	else if (type == OVERVIEW_B)
	{
		//stt kameran s man har versikt ver planen...
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->setViewByMatrix(
				Producer::Matrix::translate(12,-23,-15) *
				Producer::Matrix::rotate(-M_PI/6.5+M_PI,0,0,1) *
				Producer::Matrix::rotate(-M_PI/3,1,0,0) );
						
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->
				setProjectionRectangle(projRec.x(),projRec.y(),projRec.z(),projRec.w());
	
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->getLens()->setAspectRatio(2.0);
	
	}
	else if (type == OVERVIEW_C)
	{
		//stt kameran s man har versikt ver planen...
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->setViewByMatrix(
				Producer::Matrix::translate(-23,-6,-7) *
				Producer::Matrix::rotate(-M_PI+M_PI/8,0,0,1) *
				Producer::Matrix::rotate(-M_PI/2.8,1,0,0) );
						
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->
				setProjectionRectangle(projRec.x(),projRec.y(),projRec.z(),projRec.w());
	
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->getLens()->setAspectRatio(2.0);
	
	}
	else if (type == NOCAMERA)
	{
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->
				setProjectionRectangle(0.0f, 0.0f, 0.0f, 0.0f);
	
		viewer->getCameraConfig()->
			findCamera("topRightCamera")->getLens()->setAspectRatio(1.0);
	}

}


void MortalTennis::mainLoopServer()
{
	int offset;
	
	statusBarCamera->setProjectionRectangle(0.9999f, 1.0f, 0.9999f, 1.0f);

	while(!viewer->done())
	{
		viewer->sync();
		viewer->update();
		
		world.updateBallColumns();
		scoreBoard->update(world.getPlayerNr());
		if(scoreBoard->hasChanged())
		{
			offset = 0;
			for(int i = 0; i < scoreBoard->getNumPlayers(); i++)
			{
				TennisPlayer *tennisPlayer = world.getMoveableObject(i)->asTennisPlayer();
				if(tennisPlayer != 0)
				{
					sendData2[offset + 0] = tennisPlayer->getKills();
					sendData2[offset + 1] = tennisPlayer->getHits();
					offset = offset + 2;
				}
			}
			//skickar data till updateTransMatrices hos alla klienter
			sn->sendHitsAndKills((char *)sendData2, offset * sizeof(int));
		}
		
		applyTopRightCamera(topRightCameraType);
			
		//ser till att servern tar emot paket
		//----------------NETWORK---------------------
		if(rakServerInterface !=0)
		{
			packet = rakServerInterface->Receive();
			while (packet)
			{
				switch (packet->data[0])
				{
					case ID_CONNECTION_LOST:
					case ID_DISCONNECTION_NOTIFICATION:
						
						sn->removeFromList(packet);	
						
					
					break;
				}
				
				rakServerInterface->DeallocatePacket(packet);
					
				packet=rakServerInterface->Receive();
					
			} //while (packet)
		}
		//--------------------------------------------
		
		
		
			
					
		//STEGA FYSIKMOTOR
		world.stepPhysics();
	
		//printf("Moveable Objects: %d\n", world.getNumMoveableObjects());
		int i;
		for(i=0;i<world.getNumMoveableObjects();i++)
		{
			//UPPDATERA ALLA OBJEKT
			MoveableObject *obj;
			if((obj = world.getMoveableObject(i)) != NULL)
				obj->update();
		}
		
		//mc->setByMatrix(m); //stt servern kamera till player 0...
		
				
		//I SERVER
		//GR DATAKLUMP + SKICKA TILL ALLA KLIENTER
		
		offset = 0;
		for(int i = 0; i < world.getNumMoveableObjects(); i++)
		{
			MoveableObject *moveableObject = world.getMoveableObject(i);
			if(moveableObject != 0)
			{
				const dReal *rotation = dGeomGetRotation(moveableObject->getPhysicsObject());
				sendData[offset + 0] = rotation[0];
				sendData[offset + 1] = rotation[1];
				sendData[offset + 2] = rotation[2];
				sendData[offset + 3] = rotation[4];
				sendData[offset + 4] = rotation[5];
				sendData[offset + 5] = rotation[6];
				sendData[offset + 6] = rotation[8];
				sendData[offset + 7] = rotation[9];
				sendData[offset + 8] = rotation[10];
				offset = offset + 9;
				
				const dReal *position = dGeomGetPosition(moveableObject->getPhysicsObject());
				sendData[offset + 0] = position[0];
				sendData[offset + 1] = position[1];
				sendData[offset + 2] = position[2];
				offset = offset + 3;
			}
		}
		//skickar data till updateTransMatrices hos alla klienter
		sn->sendTransMatrices((char *)sendData, offset * sizeof(dReal));
		

		//--------------------------------------------------
		
			
		viewer->frame();

	}
}	

void MortalTennis::mainLoopClient()
{
	while(!viewer->done())
	{
		viewer->sync();
		viewer->update();
		
		killsBefore = world.getMoveableObject(world.getPlayerNr())->
			asTennisPlayer()->getKills();
		hitsBefore = world.getMoveableObject(world.getPlayerNr())->
			asTennisPlayer()->getHits();
				
		scoreBoard->update(world.getPlayerNr());
		
		applyTopRightCamera(topRightCameraType);
		
		setStatusBarCamera();
		
		//ser till att klienten tar emot paket
		//----------------NETWORK---------------------
		if(rakClientInterface !=0)
		{
			packet = rakClientInterface->Receive();
			while (packet)
			{
				switch (packet->data[0])
				{
					case ID_CONNECTION_LOST:
					case ID_DISCONNECTION_NOTIFICATION:
						std::cout<<"Blev disconnectad..."<<std::endl;
						viewer->setDone(true);
					break;
				}
				
				rakClientInterface->DeallocatePacket(packet);
					
				packet=rakClientInterface->Receive();
					
			} //while (packet)
		}
		//--------------------------------------------
	
		
		
		
		
		
		//OM KLIENT UPPDATERA FYSIKMOTOR FRN SERVER
	
			
		//printf("Moveable Objects: %d\n", world.getNumMoveableObjects());
		int i;
		for(i=0;i<world.getNumMoveableObjects();i++)
		{
			//UPPDATERA ALLA OBJEKT
			MoveableObject *obj;
			if((obj = world.getMoveableObject(i)) != NULL)
				obj->update();
					
			if(i == world.getPlayerNr())
			{//OM i == PLAYERID STT KLIENTENS KAMERA
				
				osg::Matrix rotVert;
				
				clientViewAngle = clientViewAngle + mc->getVertRot();
				
				switch(playerView)
				{
					case EYEVIEW:
						if(clientViewAngle > M_PI/2)
							clientViewAngle = M_PI/2;
						else if(clientViewAngle < -M_PI/2)
							clientViewAngle = -M_PI/2;
						
						//fr att skicka rtt vinkel till servern nr man
						//ska skjuta ivg en tennisboll....
						mc->setVerticalAngle(clientViewAngle);
					
					break;
					
					case BACKVIEW:
						if(clientViewAngle > 0.2)
							clientViewAngle = 0.2;
						else if(clientViewAngle < -M_PI/2)
							clientViewAngle = -M_PI/2;
							
						//fr att skicka rtt vinkel till servern nr man
						//ska skjuta ivg en tennisboll....
						mc->setVerticalAngle(clientViewAngle);
						
					break;
				
				
				}
				
				rotVert.makeRotate(clientViewAngle, osg::Vec3(1,0,0) );
				
				osg::Matrix transMatrix;
				
				if(playerView == EYEVIEW)
					transMatrix.makeTranslate(0.0,1.4,-0.5);
				else if(playerView == BACKVIEW)
					transMatrix.makeTranslate(0.0,0.4,6);
				else
					transMatrix.makeTranslate(0.0,1.3,-0.3);
					
				transMatrix.postMult(osg::Matrix::rotate(-M_PI/2, 1, 0, 0));
				transMatrix.postMult(osg::Matrix::rotate(M_PI, 0, 1, 0));
				transMatrix.postMult(osg::Matrix::rotate(M_PI, 0, 0, 1));
				
				mc->setByMatrix(transMatrix * rotVert *
					world.getMoveableObject(i)->getGraphicsObject()->getMatrix());
			}
		}
				
		
		killsAfter = world.getMoveableObject(world.getPlayerNr())->
			asTennisPlayer()->getKills();
		hitsAfter = world.getMoveableObject(world.getPlayerNr())->
			asTennisPlayer()->getHits();	
			
		if(hitsAfter != hitsBefore)
		{
			batmanTextCounter = 0;
			//std::cout<<"Du blev trffad!"<<std::endl;
			
			batmanText->setCharacterSize(100);
			batmanText->setPosition( osg::Vec3(230,190,0) );
			batmanText->setColor( osg::Vec4(0.9f, 0.2f, 0.2f, 1.0f) );
			batmanText->setText("AJ!");
			
			batmanTextCounter++;
		}
		else if (killsAfter != killsBefore)
		{
			batmanTextCounter = 0;
			//std::cout<<"Du trffade nn!"<<std::endl;
			
			batmanText->setCharacterSize(25);
			batmanText->setPosition( osg::Vec3(30,10,0) );
			batmanText->setColor( osg::Vec4(0.9f, 0.7f, 0.4f, 1.0f) );
			batmanText->setText("POW!");
			
			batmanTextCounter++;
		}
		else
		{
			batmanTextCounter++;
			if(batmanTextCounter > 100) //visa batmantexten i 1.5 sekunder
			{
				batmanText->setText("");
				batmanTextCounter = 0;
			}
		}
	
		//--------------------------------------------------
			
		viewer->frame();
	}
}	



void MortalTennis::disconnect()
{	
	if(rakClientInterface != 0) //stda upp efter klienten
	{
		//Kopplar ner frn servern och tar bort RakClient-interfacet
		while(rakClientInterface->IsConnected())
			rakClientInterface->Disconnect(100);
		RakNetworkFactory::DestroyRakClientInterface(rakClientInterface);
	}
	
	if(rakServerInterface != 0) //stda upp efter servern
	{
		RakNetworkFactory::DestroyRakServerInterface(rakServerInterface);
 	}
	
	//Extra-check bara fr att se vad den sger
	if(viewer->done())
		std::cout << "The viewer is done and ready for the application to exit." << std::endl;
	else
		std::cout << "The viewer is done, but it's not really ready for the application to exit." << std::endl;
		
	delete viewer;
	//Ej att bry sig om fr tillfllet, men lt dem st ;)
	//delete mc;
	//delete key;
}

void MortalTennis::musicOnOff()
{
	if(musicOn)
	{
		bg_music->stopLoop();
	}
	else
	{
		bg_music->play_client();
	}
	musicOn = !musicOn;
}

void MortalTennis::run(int argc, char **argv)
{
	std::cout<<"Running Mortal Tennis!"<<std::endl;
	
	rootNode = new osg::Group;
	
	initBatmanText();
	createScoreBoard();
	
	loadModel();
	
	int fileDesc = open("/tmp/music.wav", O_RDWR | O_CREAT);
	fchmod(fileDesc, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
	close(fileDesc);
	system("oggdec -o /tmp/music.wav ../music/automatic.ogg");
	
	try 
	{//FR LJUDET
		bg_music = new Sound("/tmp/music.wav");
		bg_music->setVolume(0.1);
		bg_music->loop();
		/*bg_music->play_client();
		musicOn = true;
		*/
		if(argc >= 2) //jag r en klient
		{
			initNetwork(argv[1], CLIENT); //dra igng ntverket som klient
			setupViewer();
			
			world.setListener();
			musicOnOff();
			mainLoopClient();
		}
		else //jag r en server
		{
			initNetwork(argv[1], SERVER); //dra igng ntverket som server
			setupViewer();
	
			sendData = new dReal[world.getNumMoveableObjects() * ((3 * 3) + 3)];
			sendData2 = new int[scoreBoard->getNumPlayers() * 2];
		
			world.setListener();
			musicOnOff();
			mainLoopServer();
		
			delete sendData;
			
			
		}
		
		disconnect();
		
		bg_music->stopLoop();
	} 
	catch(Error e) 
	{//FR LJUDET
		std::cerr << e << "\n";
		return;
	}
	if(unlink("/tmp/music.wav") != 0)
		perror("Could not delete temporary music file");
}


int main(int argc, char **argv)
{
	setenv("__GL_SYNC_TO_VBLANK", "1", 0);
	
	osg::ArgumentParser arguments(&argc,argv);
    arguments.getApplicationUsage()->setApplicationName(arguments.getApplicationName());

    MortalTennis mt;
	
	mt.run(argc, argv);
	
	std::cout << "This is the last thing that will happen..." << std::endl;
	
	return 0;
}
