#ifndef _MT_MORTALTENNIS_
#define _MT_MORTALTENNIS_

#include <stdio.h>
#include <iostream>
#include <unistd.h>

#include <osg/Vec3>
#include <osg/Matrix>
#include <osg/Node>
#include <osg/Timer>
#include <osg/Projection>
#include <osgGA/MatrixManipulator>
#include <osgDB/ReadFile>
#include <osgUtil/Optimizer>
#include <osgProducer/Viewer>
#include <osgText/Font>
#include <osgText/Text>

#include "World.h"
#include "MoveCameraManipulator.h"
#include "Network.h"
#include "IOHandler.h"
#include "CreateWorldObjectsVisitor.h"
class ScoreBoard;

#include <ode/ode.h>

#define SERVER 2000
#define CLIENT 2001

#define SCOREBOARD 3000
#define OVERVIEW_A 3001
#define OVERVIEW_B 3002
#define OVERVIEW_C 3003
#define NOCAMERA 3004

#define EYEVIEW 4000
#define BACKVIEW 4001



class MortalTennis
{
	private:
	
		World world;
		RakClientInterface *rakClientInterface;
		RakServerInterface *rakServerInterface;
		Packet *packet;
		Producer::RenderSurface *renderSurface;
		Producer::Camera* statusBarCamera;
		osgProducer::Viewer *viewer;
		MoveCameraManipulator *mc;
		osg::ref_ptr<osg::Node> loadedModel;
		void loadModel();
		void setupViewer();
		void mainLoopServer();
		void mainLoopClient();
		void disconnect();
		
		ClientNetwork *cn;
		ServerNetwork *sn;
		
		dReal *sendData;
		int *sendData2;
		
		osg::Group* rootNode;
		ScoreBoard *scoreBoard;
		bool scoreBoardOn;
		
		int topRightCameraType;
		int playerView;
				
		double clientViewAngle;
		
		int killsBefore, killsAfter, batmanTextCounter, hitsBefore, hitsAfter;
		osgText::Text* batmanText;
		osgText::Text* powerText;
		
		Sound *bg_music;
		bool musicOn;
		
	public:
				
		MortalTennis();
		void initNetwork(char *serverName, int type);
		void run(int argc, char **argv);
		Producer::CameraConfig* setupCamera();
		void createScoreBoard();
		void setTopRightCameraType(int type);
		void applyTopRightCamera(int type);
		void setPlayerView(int view);
		void initBatmanText();
		void setStatusBarCamera();
		void musicOnOff();
		osgText::Text* getBatmanText();
		osgText::Text* getPowerText();
		
};

#endif
