#include "MoveCameraManipulator.h"
#include "Network.h"
#include "MortalTennis.h"


///////////////////////////////////////
//////// MoveCameraManipulator ////////
///////////////////////////////////////

bool MoveCameraManipulator::handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa )
{
	if ( ea.getEventType() == osgGA::GUIEventAdapter::KEYDOWN ) 
	{
		switch ( ea.getKey() ) 
		{
			case 'e':
			case 'E':
				m_direction.y() = 1.0;
				goForward = true;
				keyBoard = true;
				break;

			case 'd':
			case 'D':
				m_direction.y() = -1.0;
				goBackward = true;
				keyBoard = true;
				break;

			case 's':
			case 'S':
				m_direction.x() = -1.0;
				goLeft = true;
				keyBoard = true;
				break;

			case 'f':
			case 'F':
				m_direction.x() = 1.0;
				goRight = true;
				keyBoard = true;
				break;

			default:
				return false;
		}

		return true;
	}

	if ( ea.getEventType() == osgGA::GUIEventAdapter::KEYUP ) 
	{
		switch ( ea.getKey() ) 
		{
			case 'e':
			case 'E':
			case 'd':
			case 'D':
				m_direction.y() = 0.0;
				break;

			case 's':
			case 'S':
			case 'f':
			case 'F':
				m_direction.x() = 0.0;
				break;

			default:
				return false;
		}
		
		goForward = goBackward = goLeft = goRight = false;
		if(m_direction.y() == 0.0 && m_direction.x() == 0.0)
			keyBoard = false;

		return true;
	}
	
	if ( ea.getEventType() == osgGA::GUIEventAdapter::MOVE || 
		ea.getEventType() == osgGA::GUIEventAdapter::DRAG ) 
	{
		if ( ea.getY() == 0.0 && ea.getX() == 0.0 )
			return false;

		old_horizontal_rotation = m_horizontal_rotation;
		old_vertical_rotation = m_vertical_rotation;
		
		m_horizontal_rotation += ea.getX() * m_mouse_acceleration;
		m_vertical_rotation -= ea.getY() * m_mouse_acceleration;

		m_rotation =
			osg::Matrix::rotate( m_horizontal_rotation, osg::Vec3(0,0,1) ) *
			osg::Matrix::rotate( m_vertical_rotation, osg::Vec3(1,0,0) );

		aa.requestWarpPointer(0,0);
	
		moveMouse = true;
	}
	
	if (ea.getEventType() == osgGA::GUIEventAdapter::PUSH)
		if(ea.getButtonMask() == osgGA::GUIEventAdapter::LEFT_MOUSE_BUTTON)
		{
			mousePress = true;
			gettimeofday(&beforetime, NULL);
		}
		
	if (ea.getEventType() == osgGA::GUIEventAdapter::RELEASE && mousePress)
	{
		gettimeofday(&aftertime, NULL);
		
		if(beforetime.tv_usec <= aftertime.tv_usec)
			looptime = ((long long int)(aftertime.tv_sec - beforetime.tv_sec) * (long long int)1000000000) + ((long long int)(aftertime.tv_usec - beforetime.tv_usec) * (long long int)1000);
		else
			looptime = ((long long int)(aftertime.tv_sec - beforetime.tv_sec - 1) * (long long int)1000000000) + ((long long int)(1000000 - ((aftertime.tv_usec - beforetime.tv_usec)*(-1))) * (long long int)1000);

		pressTime = looptime/5000000;
		if(pressTime <= 0)
			pressTime = 1;
						
		mouseLeft = true;
		mousePress = false;
		mortalT->getPowerText()->setText("");
	}
	
	if ( ea.getEventType() == osgGA::GUIEventAdapter::FRAME )
	{
		if(mousePress)
		{
			gettimeofday(&aftertime, NULL);
		
			if(beforetime.tv_usec <= aftertime.tv_usec)
				looptime2 = ((long long int)(aftertime.tv_sec - beforetime.tv_sec) * (long long int)1000000000) + ((long long int)(aftertime.tv_usec - beforetime.tv_usec) * (long long int)1000);
			else
				looptime2 = ((long long int)(aftertime.tv_sec - beforetime.tv_sec - 1) * (long long int)1000000000) + ((long long int)(1000000 - ((aftertime.tv_usec - beforetime.tv_usec)*(-1))) * (long long int)1000);

			pressTime2 = looptime2/5000000;
			
			mortalT->getPowerText()->setCharacterSize(20);
			mortalT->getPowerText()->setPosition( osg::Vec3(520,5,0) );
			
			if(pressTime2 < 50)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.80f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("|         |");
			}
			else if(pressTime2 < 100)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.72f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("||        |");
			}
			else if(pressTime2 < 150)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.64f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("|||       |");
			}
			else if(pressTime2 < 200)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.56f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("||||      |");
			}
			else if(pressTime2 < 250)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.48f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("|||||     |");
			}
			else if(pressTime2 < 300)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.40f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("||||||    |");
			}
			else if(pressTime2 < 350)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.32f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("|||||||   |");
			}
			else if(pressTime2 < 400)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.24f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("||||||||  |");
			}
			else if(pressTime2 < 450)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.16f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("||||||||| |");
			}
			else if(pressTime2 > 450)
			{
				mortalT->getPowerText()->setColor( osg::Vec4(1.0f, 0.08f, 0.0f, 1.0f) );
				mortalT->getPowerText()->setText("|||||||||||");
			}
			
		}		
			
		calcMatrix(); 	
	}


	return false;
}

void MoveCameraManipulator::sendVectorToServer(osg::Vec3 v, int type)
{
	switch(type)
	{
		case WALK_VEC:
			//fr att skicka vektorer till fysik-servern:
			if(rakClientInterface != 0)
				rakClientInterface->RPC("updatePlayerWalkVec", (char*)&v, 
					sizeof(osg::Vec3)*8, HIGH_PRIORITY,  UNRELIABLE_SEQUENCED, 0, false);
		break;
		
		case SIGHT_VEC:
			//fr att skicka vektorer till fysik-servern:
			if(rakClientInterface != 0)
				rakClientInterface->RPC("updatePlayerSightVec", (char*)&v, 
					sizeof(osg::Vec3)*8, HIGH_PRIORITY,  RELIABLE_ORDERED, 0, false);
		break;
		
		case ROTATION_VEC:
			//fr att skicka vektorer till fysik-servern:
			if(rakClientInterface != 0)
				rakClientInterface->RPC("updatePlayerRotationVec", (char*)&v, 
					sizeof(osg::Vec3)*8, HIGH_PRIORITY,  UNRELIABLE_SEQUENCED, 0, false);
		break;
		
		
		default:
		break;
	}

}

double MoveCameraManipulator::getVertRot()
{
	return clientViewAngle;
}

void MoveCameraManipulator::checkIO()
{
	if(keyBoard || moveMouse)
	{
		//skicka datan till fysik-servern
		if(keyBoard) 
			sendVectorToServer(m_direction, WALK_VEC);
		
		if(moveMouse)
		{
			osg::Vec3 walkDirection;
			walkDirection._v[0] = old_horizontal_rotation -  m_horizontal_rotation;
			walkDirection._v[1] = old_vertical_rotation -  m_vertical_rotation;
			walkDirection._v[2] = 0;
			sendVectorToServer(walkDirection, ROTATION_VEC);
		
			clientViewAngle = walkDirection.y();
		}
					
		moveMouse = false;
	}
	if(mouseLeft)
	{
		//skicka datan till fysik-servern
		osg::Vec3 sightVector;
		sightVector._v[0] = cVerticalAngle;
		sightVector._v[1] = pressTime;
		sightVector._v[2] = 0;
		sendVectorToServer(sightVector, SIGHT_VEC);
		mouseLeft = false;
	}
}

void MoveCameraManipulator::calcMatrix()
{
	clientViewAngle = 0.0;
	
	if ( m_direction.length() > 0.01 )
		m_position = m_position + m_rotation * m_direction *
			m_curr_vel * m_timer.delta_s( m_last_frame, m_timer.tick() );

	//vektor t det hllet man siktar...
	sightDirection = m_rotation * osg::Vec3(0,1,0);
    
	//Kollar ifall vi har rrt oss o uppdatera i sdana
	//fall vektorena som ska skickas till server
	if(keyBoard || mouseLeft || moveMouse)
		checkIO();

	if(!cameraManually)
	{
		osg::Vec3 up = m_rotation * osg::Vec3(0,0,1);
		osg::Vec3 center = m_position + osg::Vec3(0,0,m_height);
		osg::Vec3 eye = center - sightDirection * m_distance;

		m_matrix.makeLookAt(eye,center,up);
		m_matrix = osg::Matrix::inverse(m_matrix);

		//rotera rtt i vrlden...
		//osg::Matrix m;
		//m.makeRotate(-M_PI/2.0,1,0,0);
		//m_matrix = m_matrix*m;

	}
	m_last_frame = m_timer.tick();
}

void MoveCameraManipulator::setVerticalAngle(double angle)
{
	cVerticalAngle = angle;
}
