#ifndef _MT_MOVECAMERAMANIPULATOR_
#define _MT_MOVECAMERAMANIPULATOR_

//#include "MortalTennis.h"

#include <osg/Vec3>
#include <osg/Matrix>
#include <osg/Node>
#include <osg/Timer>
#include <osgGA/MatrixManipulator>
#include <osgDB/ReadFile>
#include <osgUtil/Optimizer>
#include <osgProducer/Viewer>
#include <time.h>

class RakClientInterface;
class MortalTennis;

#define WALK_VEC 1000
#define SIGHT_VEC 1001
#define ROTATION_VEC 1002


///////////////////////////////////////
//////// MoveCameraManipulator ////////
///////////////////////////////////////

class MoveCameraManipulator : public osgGA::MatrixManipulator 
{
	private:
	
		osg::Matrix m_matrix;
		osg::Matrix m_rotation;
		osg::Vec3 m_direction;
		osg::Vec3 m_position;
		osg::Vec3 sightDirection;
	
		double m_distance;
		double m_curr_vel;
		double m_height;
		double m_horizontal_rotation;
		double m_vertical_rotation;
		double m_mouse_acceleration;
	
		osg::ref_ptr<osg::Node> m_node;
	
		osg::Timer m_timer;
		osg::Timer_t m_last_frame;
	
		bool goForward, goBackward, goLeft, goRight;
		bool keyBoard, mouseLeft, moveMouse, mousePress;

		bool cameraManually;

		void checkIO();
		void sendVectorToServer(osg::Vec3 v, int type);
		
		RakClientInterface *rakClientInterface;
		MortalTennis *mortalT;
		
		double old_horizontal_rotation;
		double old_vertical_rotation;
		
		double clientViewAngle;
		double cVerticalAngle;
		
		struct timeval beforetime;
		struct timeval aftertime;
		
		long long int looptime;
		long long int looptime2;
		
		int pressTime, pressTime2;
				
	public:
	
		MoveCameraManipulator(RakClientInterface *rci, MortalTennis *mt): // konstruktorn
			m_distance(0.01), m_curr_vel(3.0), m_height(12.0),
			m_horizontal_rotation(-M_PI/2.0), m_vertical_rotation(-M_PI/2.0),
			m_mouse_acceleration(1.0), m_last_frame(m_timer.tick()),
			keyBoard(false), mouseLeft(false), cameraManually(false),
			old_horizontal_rotation(0.0), old_vertical_rotation(0.0),
			clientViewAngle(0.0), cVerticalAngle(0.0)
		{
			rakClientInterface = rci;
			mortalT = mt;
		};

		virtual bool handle( const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa );

		double getVertRot();
		
		void setVerticalAngle(double angle);

		void setCameraManually(bool c)
		{
			cameraManually = c;	
		}

		void calcMatrix();

		/** set the position of the matrix manipulator using a 4x4 Matrix.*/
		virtual void setByMatrix(const osg::Matrixd& matrix) 
		{ 
			m_matrix = matrix; 
		}

		/** set the position of the matrix manipulator using a 4x4 Matrix.*/
		virtual void setByInverseMatrix(const osg::Matrixd& matrix) 
		{ 
			m_matrix = matrix; 
		}

		/** get the position of the manipulator as 4x4 Matrix.*/
		virtual osg::Matrixd getMatrix() const 
		{ 
			return m_matrix; 
		}

		/** get the position of the manipulator as a inverse matrix of the manipulator, typically used as a model view matrix.*/
		virtual osg::Matrixd getInverseMatrix() const 
		{ 
			return osg::Matrix::inverse(m_matrix);  
		}

};

#endif
