#include "Network.h"
#include "MoveCameraManipulator.h"
#include "ScoreBoard.h"
#include <vector>
#include <string>
#include <algorithm>
#include <openalpp/source.h>


World *world;
osgProducer::Viewer *viewer;
MoveCameraManipulator *moveCameraManipulator;
ScoreBoard *scoreBoard;

int players;
PlayerID pID[MAX_PLAYERS];
bool pList[MAX_PLAYERS];

RakServerInterface *rakServerInterface;
RakClientInterface *rakClientInterface;

std::vector<openalpp::Source *> sounds;
std::vector<std::string> soundFileNames;


///////////////
// KLIENTENS //
///////////////

void playSound(char *input, int numberOfBitsOfData, PlayerID sender)
{
	float *soundPosition = (float *)input;
	float *soundGain = (float *) &input[3 * sizeof(float)];
	char *soundAmbient = &input[4 * sizeof(float)];
	char *soundFileName = &input[4 * sizeof(float) + 1];
	
	openalpp::Source *sound;
	
	//Tar ut spelarpositionen, behvs oavsett fr att stta positionen p ljudet
	//relativt till spelaren
	const dReal *playerPosition = dGeomGetPosition(world->getMoveableObject(world->getPlayerNr())->getPhysicsObject());

	osg::Vec3 position = osg::Vec3(
			soundPosition[0] - playerPosition[0],
			soundPosition[1] - playerPosition[1],
			soundPosition[2] - playerPosition[2]);

	const dReal *playerRotation = dGeomGetRotation(world->getMoveableObject(world->getPlayerNr())->getPhysicsObject());
	osg::Matrix rotationMatrix;
	rotationMatrix.ptr()[0] = playerRotation[0];//RowCol(1,1)
	rotationMatrix.ptr()[1] = playerRotation[4];//RowCol(1,2)
	rotationMatrix.ptr()[2] = playerRotation[8];//RowCol(1,3)
	rotationMatrix.ptr()[4] = playerRotation[1];//RowCol(2,1)
	rotationMatrix.ptr()[5] = playerRotation[5];//RowCol(2,2)
	rotationMatrix.ptr()[6] = playerRotation[9];//RowCol(2,3)
	rotationMatrix.ptr()[8] = playerRotation[2];//RowCol(3,1)
	rotationMatrix.ptr()[9] = playerRotation[6];//RowCol(3,2)
	rotationMatrix.ptr()[10] = playerRotation[10];//RowCol(3,3)

	position = position * rotationMatrix;
	
	//Sker igenom listan av namn p ljud fr att se om det finns inladdat
	//sen tidigare
	std::vector<std::string>::iterator soundFileNamesIterator;
	soundFileNamesIterator = find(soundFileNames.begin(), soundFileNames.end(), soundFileName);
	if(soundFileNamesIterator != soundFileNames.end())
	{
		std::vector<std::string>::difference_type soundFileNameIndex;
		soundFileNameIndex = distance(soundFileNames.begin(), soundFileNamesIterator);
		sound = sounds[soundFileNameIndex];
	}
	else
	{
		sound = new openalpp::Source(soundFileName);
		soundFileNames.push_back(std::string(soundFileName));
		sounds.push_back(sound);
	}	

	sound->setPosition(position.x(), position.y(), position.z());
	sound->setGain(*soundGain);
	sound->setAmbient(*soundAmbient);
	sound->play();

	/*std::cout << "Spelar upp "<< soundFileName << " med volym " << *soundGain; 	if(*soundAmbient)
		std::cout << " som bakgrundsljud (ambient)" << std::endl;
	else
		std::cout << " p position " << position.x() << " " << position.y() << " " << position.z() << std::endl;
*/
}

void sendPlaySound(char *soundData, int bytes)
{
	rakServerInterface->RPC("playSound", soundData, bytes * 8, HIGH_PRIORITY, UNRELIABLE_SEQUENCED, 0, UNASSIGNED_PLAYER_ID, true, false);

}

void updateHitsAndKills(char *input, int numberOfBitsOfData, PlayerID sender)
{
	int *recievedData = (int *)input;
	
	int offset = 0;
	
	for(int i = 0; i < scoreBoard->getNumPlayers(); i++)
	{
		TennisPlayer *tennisPlayer = world->getMoveableObject(i)->asTennisPlayer();
		if(tennisPlayer != 0)
		{
				tennisPlayer->setKills(recievedData[offset + 0]);
				tennisPlayer->setHits(recievedData[offset + 1]);
				offset = offset + 2;
		}
	}
	
	//std::cout << "updateHitsAndKills!" << std::endl;
}

void updateTransMatrices(char *input, int numberOfBitsOfData, PlayerID sender)
{
	dReal *recievedData = (dReal *)input;
	
	int offset = 0;	
	dMatrix3 rotationMatrix;
	

	for(int i = 0; i < world->getNumMoveableObjects(); i++)
	{
		MoveableObject *moveableObject = world->getMoveableObject(i);
		if(moveableObject != 0)
		{
			//Stter rotationen fr objektet i fysikmotorn
			rotationMatrix[0] = recievedData[offset + 0];
			rotationMatrix[1] = recievedData[offset + 1];
			rotationMatrix[2] = recievedData[offset + 2];
			rotationMatrix[3] = 0;
			rotationMatrix[4] = recievedData[offset + 3];
			rotationMatrix[5] = recievedData[offset + 4];
			rotationMatrix[6] = recievedData[offset + 5];
			rotationMatrix[7] = 0;
			rotationMatrix[8] = recievedData[offset + 6];
			rotationMatrix[9] = recievedData[offset + 7];
			rotationMatrix[10] = recievedData[offset + 8];
			rotationMatrix[11] = 0;
			dGeomSetRotation(moveableObject->getPhysicsObject(), rotationMatrix);
			offset = offset + 9;
			
			//Stter positionen fr objektet i fyskmotorn
			dGeomSetPosition(moveableObject->getPhysicsObject(),
				recievedData[offset + 0],
				recievedData[offset + 1],
				recievedData[offset + 2]);
			offset = offset + 3;
		}
	}
}

void handShakeAccepted(char *input, int numberOfBitsOfData, PlayerID sender)
{
	std::cout<<"Jag fick vara med o leka! Jag r spelare: "<<*((int*)input)<<std::endl;
	world->setPlayerNr(*((int*)input));
}

void handShakeNotAccepted(char *input, int numberOfBitsOfData, PlayerID sender)
{
	std::cout<<"NJ! Jag fr inte vara med o leka :((((("<<std::endl;
	viewer->setDone(true);
}



//////////////
// SERVERNS //
//////////////
void updatePlayerRotationVec(char *input, int numberOfBitsOfData, PlayerID s)
{
	osg::Vec3 *rotationVector = (osg::Vec3 *)input;

	rotationVector->_v[2] = 0;
	
	for(int i = 0; i < 8; i++)
	{
		if(s == pID[i])
		{
			MoveableObject *moveableObject = world->getMoveableObject(i);
			if(moveableObject != 0)
			{
				osg::Matrix rotationMatrix;
				//Uppdatera Rotation
				const dReal *rotation = dGeomGetRotation(moveableObject->getPhysicsObject());
				rotationMatrix.ptr()[0] = rotation[0];//RowCol(1,1)
				rotationMatrix.ptr()[1] = rotation[4];//RowCol(1,2)
				rotationMatrix.ptr()[2] = rotation[8];//RowCol(1,3)
				rotationMatrix.ptr()[4] = rotation[1];//RowCol(2,1)
				rotationMatrix.ptr()[5] = rotation[5];//RowCol(2,2)
				rotationMatrix.ptr()[6] = rotation[9];//RowCol(2,3)
				rotationMatrix.ptr()[8] = rotation[2];//RowCol(3,1)
				rotationMatrix.ptr()[9] = rotation[6];//RowCol(3,2)
				rotationMatrix.ptr()[10] = rotation[10];//RowCol(3,3)
				
				osg::Matrix horizontalRotation;
				osg::Matrix verticalRotation;
				horizontalRotation.makeRotate(rotationVector->x(), osg::Vec3(0,0,1));
								
				rotationMatrix.preMult(horizontalRotation);
				
				dMatrix3 odeRotation;
				
				odeRotation[0] = rotationMatrix.ptr()[0];
				odeRotation[1] = rotationMatrix.ptr()[4];
				odeRotation[2] = rotationMatrix.ptr()[8];
				odeRotation[3] = 0;
				odeRotation[4] = rotationMatrix.ptr()[1];
				odeRotation[5] = rotationMatrix.ptr()[5];
				odeRotation[6] = rotationMatrix.ptr()[9];
				odeRotation[7] = 0;
				odeRotation[8] = rotationMatrix.ptr()[2];
				odeRotation[9] = rotationMatrix.ptr()[6];
				odeRotation[10] = rotationMatrix.ptr()[10];
				odeRotation[11] = 0;
				
				dGeomSetRotation(moveableObject->getPhysicsObject(), odeRotation);
			}
		}
	}
}

void updatePlayerWalkVec(char *input, int numberOfBitsOfData, PlayerID s)
{
	osg::Vec3 *walkVector = (osg::Vec3*)input;
	//std::cout <<"walkVector: "<< ((osg::Vec3*)input)->x() << " " << ((osg::Vec3*)input)->y() << std::endl;

	walkVector->_v[1] *= -1;

	for(int i = 0; i < 8; i++)
	{
		if(s == pID[i])
		{
			MoveableObject *moveableObject = world->getMoveableObject(i);
			if(moveableObject != 0)
			{
				osg::Matrix rotationMatrix;
				const dReal *rotation = dGeomGetRotation(moveableObject->getPhysicsObject());
				rotationMatrix.ptr()[0] = rotation[0];//RowCol(1,1)
				rotationMatrix.ptr()[1] = rotation[4];//RowCol(1,2)
				rotationMatrix.ptr()[2] = rotation[8];//RowCol(1,3)
				rotationMatrix.ptr()[4] = rotation[1];//RowCol(2,1)
				rotationMatrix.ptr()[5] = rotation[5];//RowCol(2,2)
				rotationMatrix.ptr()[6] = rotation[9];//RowCol(2,3)
				rotationMatrix.ptr()[8] = rotation[2];//RowCol(3,1)
				rotationMatrix.ptr()[9] = rotation[6];//RowCol(3,2)
				rotationMatrix.ptr()[10] = rotation[10];//RowCol(3,3)
			
				walkVector->_v[2] = 0;
				
				*walkVector = rotationMatrix * *walkVector;
				
				*walkVector = *walkVector * 0.1;

				dReal newPosition[3];
				const dReal *position = dGeomGetPosition(moveableObject->getPhysicsObject());
				newPosition[0] = position[0] + walkVector->x();
				newPosition[1] = position[1] - walkVector->y();
				newPosition[2] = position[2];
				
				TennisPlayer *player = moveableObject->asTennisPlayer();
				player->move(*walkVector);
				
				dGeomSetPosition(moveableObject->getPhysicsObject(), newPosition[0], newPosition[1], newPosition[2]);
				
				//std::cout << "Player position: " << newPosition[0] << " " << newPosition[1] << " " << newPosition[2] << std::endl;
			}
		}
	}
}

void updatePlayerSightVec(char *input, int numberOfBitsOfData, PlayerID s)
{
	osg::Vec3 *sightVector = (osg::Vec3*)input;

	//std::cout<<"time keydown: "<<sightVector->_v[1]<<std::endl;

	for(int i = 0; i < 8; i++)
	{
		if(s == pID[i])
		{
			MoveableObject *moveableObject = world->getMoveableObject(i);
			if(moveableObject != 0)
			{
				osg::Matrix rotationMatrix;
				osg::Matrix verticalRotationMatrix;
				osg::Matrix horizontalRotationMatrix;
				
				verticalRotationMatrix.makeRotate(sightVector->x(), 1, 0, 0);
				const dReal *odeRotation = dGeomGetRotation(moveableObject->getPhysicsObject());
				horizontalRotationMatrix.ptr()[0] = odeRotation[0];//RowCol(1,1)
				horizontalRotationMatrix.ptr()[1] = odeRotation[4];//RowCol(1,2)
				horizontalRotationMatrix.ptr()[2] = odeRotation[8];//RowCol(1,3)
				horizontalRotationMatrix.ptr()[4] = odeRotation[1];//RowCol(2,1)
				horizontalRotationMatrix.ptr()[5] = odeRotation[5];//RowCol(2,2)
				horizontalRotationMatrix.ptr()[6] = odeRotation[9];//RowCol(2,3)
				horizontalRotationMatrix.ptr()[8] = odeRotation[2];//RowCol(3,1)
				horizontalRotationMatrix.ptr()[9] = odeRotation[6];//RowCol(3,2)
				horizontalRotationMatrix.ptr()[10] = odeRotation[10];//RowCol(3,3)
				
				rotationMatrix = verticalRotationMatrix * horizontalRotationMatrix;
				
				osg::Vec3 ballDirection;
				
				if(sightVector->_v[1] > 450)
					sightVector->_v[1] = 450;
					
				if(sightVector->_v[1] < 50)
					sightVector->_v[1] = 50;
				
				ballDirection._v[0] = 0;
				ballDirection._v[1] = sightVector->y();
				ballDirection._v[2] = sightVector->y()/10;
				
				ballDirection = ballDirection * rotationMatrix;
								
				moveableObject->asTennisPlayer()->shootBall(ballDirection);
			}
		}
	}

}

void handShake(char *input, int numberOfBitsOfData, PlayerID s)
{
	int playerNr;
	std::cout <<"handShake: "<< input << std::endl;
	if (players < MAX_PLAYERS)
	{
		for (int i=1; i<MAX_PLAYERS; i++)
		{
			if(pList[i] == false)
			{
				pID[i] = s;	
				pList[i] = true;
				playerNr = i; //mellan 1 och 7
				i = MAX_PLAYERS;
			}
		
		}
		
		players++;
	
		//std::cout<<"Du fr vara med o leka min vn..."<<std::endl;
		std::cout<<"Player "<<playerNr<<" connected"<<std::endl;
		
		world->getMoveableObject(playerNr)->asTennisPlayer()->activate(true);
		
		rakServerInterface->RPC("handShakeAccepted", (char*)&playerNr, sizeof(int)*8, HIGH_PRIORITY, RELIABLE_ORDERED, 0, s, false, false);
		scoreBoard->setChanged(true);
	}
	else
	{
		std::cout<<"Du fr inte vara med! HAHA!"<<std::endl;
	
		rakServerInterface->RPC("handShakeNotAccepted", (char*)&players, sizeof(int)*8, HIGH_PRIORITY, RELIABLE_ORDERED, 0, s, false, false);
		
	}
}




///////////////////////////////
//////// ClientNetwork ////////
///////////////////////////////

ClientNetwork::ClientNetwork(World *w, osgProducer::Viewer *v, MoveCameraManipulator *mc, ScoreBoard *sb)
{
	world = w;
	viewer = v;
	moveCameraManipulator = mc;
	scoreBoard = sb;
}


bool ClientNetwork::init(char *serverName)
{
	rakClientInterface=RakNetworkFactory::GetRakClientInterface();
	
	if(rakClientInterface->Connect(serverName, 20370, 0, 0, 0))
	{
		REGISTER_AS_REMOTE_PROCEDURE_CALL(rakClientInterface, updateTransMatrices);
		REGISTER_AS_REMOTE_PROCEDURE_CALL(rakClientInterface, handShakeAccepted);
		REGISTER_AS_REMOTE_PROCEDURE_CALL(rakClientInterface, handShakeNotAccepted);
		REGISTER_AS_REMOTE_PROCEDURE_CALL(rakClientInterface, updateHitsAndKills);
		REGISTER_AS_REMOTE_PROCEDURE_CALL(rakClientInterface, playSound);

		usleep(100000);
		while(!rakClientInterface->IsConnected())
		{
			std::cout<<"Could not connect to server..."<<std::endl;
			//viewer->setDone(true);
			usleep(500000);
		}

		
		char str[100];
		sprintf(&str[0],"fr jag leka?");
	
		rakClientInterface->RPC("handShake", str, 
			(strlen(str)+1)*8, HIGH_PRIORITY, RELIABLE_ORDERED, 0, false);
			
		return true;
	}
	else
		return false;


}

RakClientInterface* ClientNetwork::getInterface()
{
	return rakClientInterface;
}



///////////////////////////////
//////// ServerNetwork ////////
///////////////////////////////

ServerNetwork::ServerNetwork(World *w, ScoreBoard *sb)
{
	world = w;
	scoreBoard = sb;
}


bool ServerNetwork::init()
{
	players = 0;
	for(int i=0; i<MAX_PLAYERS; i++)
		pList[i] = false;
	
	rakServerInterface=RakNetworkFactory::GetRakServerInterface();
	
	if (rakServerInterface->Start(8, 0, 0, 20370))
	{
		//std::cout << "Starting the server." << std::endl;

		REGISTER_AS_REMOTE_PROCEDURE_CALL(rakServerInterface, updatePlayerRotationVec);
		REGISTER_AS_REMOTE_PROCEDURE_CALL(rakServerInterface, updatePlayerSightVec);
		REGISTER_AS_REMOTE_PROCEDURE_CALL(rakServerInterface, updatePlayerWalkVec);
		REGISTER_AS_REMOTE_PROCEDURE_CALL(rakServerInterface, handShake);
	
		return true;
	}
	else
	{
		std::cout << "Failed to start the server." << std::endl;
		RakNetworkFactory::DestroyRakServerInterface(rakServerInterface);
		return false;
	}
}

void ServerNetwork::sendTransMatrices(char* data, int bytes)
{
	//std::cout<<"skickar data till klienterna..."<<std::endl;
	rakServerInterface->RPC("updateTransMatrices", data, bytes*8, HIGH_PRIORITY,  UNRELIABLE_SEQUENCED, 0, UNASSIGNED_PLAYER_ID, true, false);
}

void ServerNetwork::sendHitsAndKills(char *data, int bytes)
{
	rakServerInterface->RPC("updateHitsAndKills", data, bytes*8, HIGH_PRIORITY,  RELIABLE_ORDERED, 0, UNASSIGNED_PLAYER_ID, true, false);
}

RakServerInterface* ServerNetwork::getInterface()
{
	return rakServerInterface;
}


void ServerNetwork::removeFromList(Packet *packet)
{
	for (int i=0; i<MAX_PLAYERS; i++)
	{
		if (pID[i] == packet->playerId)
		{
			std::cout<<"Player "<<i<<" disconnected"<<std::endl;	
					
			//std::cout << "Tog bort spelaren ur listan..." << std::endl;
			
			world->getMoveableObject(i)->asTennisPlayer()->activate(false);
		
			pList[i] = false;
			players--;
			i = MAX_PLAYERS;
		}
	}
}
