#include "TennisPlayer.h"
#include "Ball.h"
#include "PhysicsObject.h" 
#include "Sound.h"

#include <iostream>
#include <ode/ode.h>

TennisPlayer::TennisPlayer(PhysicsObject *n_physicsObject, osg::MatrixTransform *n_graphicsObject) : MoveableObject(n_physicsObject, n_graphicsObject)
{
	//printf("Created Tennis Player.\n");
	hitCount = 0;
	kills = 0;

	bounce = 0.0;
	friction = 200;

	strcpy(name,"Player X");
	
	listener = this;
	active = false;
	
	runCount = 0;
	runningSound = new Sound("../samples/running.wav");
	runningSound->setVolume(0.02);
	runningSound->setAmbient(false);
	
	hitBallSound = new Sound("../samples/hit_ball.wav");
	hitBallSound->setAmbient(false);
	
	pickUpBallSound = new Sound("../samples/pick_up_ball.wav");
	pickUpBallSound->setAmbient(false);
	
	hitPlayerSound = new Sound("../samples/hit_tennis_player.wav");
	hitPlayerSound->setVolume(0.4);
	hitPlayerSound->setAmbient(true);
	
	tackleCount = 0;
	tackleSound = new Sound("../samples/tackle_player.wav");
	tackleSound->setVolume(0.1);
	tackleSound->setAmbient(false);
	
}

TennisPlayer::~TennisPlayer()
{
	//printf("Deleted Tennis Player.\n");
	
}
void 
TennisPlayer::kill()
{
	if(active)
	{
		kills++;
	}
}
void 
TennisPlayer::setName(char* a_name)
{
	strcpy(name,a_name);
}
void 
TennisPlayer::activate(bool val)
{
	while(getBallCount() > 0)
		shootBall(osg::Vec3(40, 40, 40));
	active = val;
}
void 
TennisPlayer::collision(MoveableObject *mo, dContact thisContact)
{
	//printf("I tennis player collision.\n");

	Ball *ball;
	TennisPlayer *otherPlayer;
	
	ball = mo->asBall();
	if(ball != 0)
	{
		//std::cout << "Det va en boll jag kolliderade med!" << std::endl;
		
		otherPlayer = ball->getLastHit()->asTennisPlayer();
		if((otherPlayer != 0) && (otherPlayer != this))
		{
			if(active)
			{
				hitCount++;
				otherPlayer->kill();
				//printf("HIT!!!!!!!!!\n\n%s has now been hit %d times and has %d kills.\n\n%s has been hit %d times and now has %d kills.\n\n", name, hitCount, kills, otherPlayer->getName(), otherPlayer->getHits(), otherPlayer->getKills());				
				hitPlayerSound->playAtPos(otherPlayer,listener);
				hitPlayerSound->play();
			}
			ball->collision(this, thisContact);
		}
		else
		{
			//hitCount++;
		
			//Kollar om farten r nog lg
			const dReal *linearVel = dBodyGetLinearVel(dGeomGetBody(ball->getPhysicsObject()));
		
			osg::Vec3 linearVelVec;
		
			linearVelVec._v[0] = linearVel[0];
			linearVelVec._v[1] = linearVel[1];
			linearVelVec._v[2] = linearVel[2];
		
			if((linearVelVec.length() < 2) && (active))
			{
				//std::cout << "Bollens hastighet r under 2, plockar upp den!" << std::endl;
				pickUpBall(ball);
			}
			else
			{
				//std::cout << "Bollens hastighet r hgre n 2, det blir en vanlig kontakt istllet!" << std::endl;
				ball->collision(this, thisContact);
			}
		
			//hitBall(ball);
		}
		
	}
	else
	{
		//printf("Hit other than ball.\n");
		
		dGeomID own_object = getPhysicsObject();
		dGeomID other_object = mo->getPhysicsObject();
	
		dJointID collisionJoint;
	
		//dContact thisContact;
		
		thisContact.surface.mode = dContactBounce;
		thisContact.surface.mu = friction * mo->getFriction();
		//thisContact.surface.mu2 = 1000;
		thisContact.surface.bounce = bounce * mo->getBounce();
		thisContact.surface.bounce_vel = 0;	
	
		collisionJoint = dJointCreateContact(physicsObject->getWorld(), physicsObject->getGroup(), &thisContact);
		
		dJointAttach(collisionJoint, dGeomGetBody(own_object), dGeomGetBody(other_object));
		
		/*TennisPlayer *otherPlayer;
		otherPlayer = mo->asTennisPlayer();
		if(otherPlayer != 0)
		{
			//Trffade annan TennisPlayer
			*/
			if(tackleCount == 0)
			{
				tackleSound->playAtPos(this,listener);
				tackleSound->play();
			}
			tackleCount++;
	
			if(tackleCount == 14)	
				tackleCount = 0;
			
		//}
	}
}
void
TennisPlayer::hitBall(Ball *ball)
{
	dGeomID ball_object = ball->getPhysicsObject();
	
	dBodySetForce(dGeomGetBody(ball_object), walkVector.x()*500, -walkVector.y()*500, 15);
	
	hitBallSound->playAtPos(this,listener);
	hitBallSound->play();
}
void 
TennisPlayer::move(osg::Vec3 the_walkVector)
{
	////printf("I tennis player move.\n");
	walkVector = the_walkVector;
	
	if(runCount == 0)
	{
		runningSound->playAtPos(this,listener);
		runningSound->play();
	}
	runCount++;
	
	if(runCount == 42)	
		runCount = 0;
}

int TennisPlayer::getBallCount(void)
{
	return ballList.size();
}

void TennisPlayer::pickUpBall(Ball *ball)
{
	dGeomSetPosition(ball->getPhysicsObject(), 0, 0, 0);
	dBodySetForce(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetTorque(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetLinearVel(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetAngularVel(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetForce(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetTorque(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dGeomSetPosition(ball->getPhysicsObject(), 0, 0, 0);
	
	dBodyDisable(dGeomGetBody(ball->getPhysicsObject()));
	
	dGeomSetPosition(ball->getPhysicsObject(), 0, 0, 0);
	dBodySetForce(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetTorque(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetLinearVel(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetAngularVel(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetForce(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dBodySetTorque(dGeomGetBody(ball->getPhysicsObject()), 0, 0, 0);
	dGeomSetPosition(ball->getPhysicsObject(), 0, 0, 0);
	
	ball->setLastHit(this);
	ballList.push_back(ball);
	
	pickUpBallSound->playAtPos(this,listener);
	pickUpBallSound->play();
}

void TennisPlayer::shootBall(osg::Vec3 direction) //VILKET NAMN! MAN SLR I TENNIS, SKJUTER GR MAN I FOTBOLL!!! ;)
//Namnet ger!! Ska implementera en tennis-trainer-maskin  skjuta bollar p
//dig i MortalTennis s du tiger och inser att man skjuter bollar :=)
//Jo isf skjuter man dem ju. Men man skjuter inte med ett RACKET!!! D SLR MAN!!!!
//Viktigpetter!! *knner att det kommer frn rtt mnniska* ;=)))
{
	if(!ballList.empty())
	{
		Ball *ball;
		ball = ballList.back();
		ballList.pop_back();

		osg::Vec3 positionOffset = osg::Vec3(0, 1.1, 1.2);
		osg::Matrix rotationMatrix;
	
		const dReal *odeRotation = dGeomGetRotation(physicsObject->getObject());
		rotationMatrix.ptr()[0] = odeRotation[0];
		rotationMatrix.ptr()[1] = odeRotation[4];
		rotationMatrix.ptr()[2] = odeRotation[8];
		rotationMatrix.ptr()[4] = odeRotation[1];
		rotationMatrix.ptr()[5] = odeRotation[5];
		rotationMatrix.ptr()[6] = odeRotation[9];
		rotationMatrix.ptr()[8] = odeRotation[2];
		rotationMatrix.ptr()[9] = odeRotation[6];
		rotationMatrix.ptr()[10] = odeRotation[10];
				
		positionOffset = positionOffset * rotationMatrix;

		const dReal *odePosition = dGeomGetPosition(physicsObject->getObject());
	
		dGeomSetPosition(ball->getPhysicsObject(),
			odePosition[0] + positionOffset.x(),
			odePosition[1] + positionOffset.y(),
			odePosition[2] + positionOffset.z());
		
		dBodySetForce(dGeomGetBody(ball->getPhysicsObject()),
			direction.x(),
			direction.y(),
			direction.z());
				
		ball->setLastHit(this);
		dBodyEnable(dGeomGetBody(ball->getPhysicsObject()));
				
		//std::cout << "SLOG ivg en boll!" << std::endl;
		
		hitBallSound->playAtPos(this,listener);
		hitBallSound->play();
		
	}
	else
	{
		//std::cout << "Fr inte SL utan bollar!" << std::endl;
	}
}


