#include "World.h"
#include "Ball.h"
#include "ScoreBoard.h"
#include "TennisPlayer.h"

World::World()
{
	physicsWorld = dWorldCreate();
	physicsSpace = dHashSpaceCreate(0);
	collisionGroup = dJointGroupCreate(0);
	dWorldSetGravity(physicsWorld, 0, 0, -9.82);
	
	numWorldObjects = 0;
	//Pekare till 0 fr att kunna gra null-check
	worldObjects = 0;
	playerNr = 0; //servern r 0 om man r klient s skrivs denna ver med 1-7
}

World::~World()
{
	if(worldObjects != 0)
		delete worldObjects;

	dWorldDestroy(physicsWorld);
	dSpaceDestroy(physicsSpace);
	dCloseODE();
	
}

dWorldID 
World::getPhysicsWorld()
{
	return physicsWorld;
}

dSpaceID 
World::getPhysicsSpace()
{
	return physicsSpace;
}

dJointGroupID 
World::getPhysicsGroup()
{
	return collisionGroup;
}
		
int 
World::getNumWorldObjects(void)
{
	return numWorldObjects;
}

WorldObject *
World::getWorldObject(int index)
{
	return worldObjects[index];
}

WorldObject **
World::getWorldObjects(void)
{
	return worldObjects;
}

int 
World::getNumMoveableObjects(void)
{
	return numMoveableObjects;
}

MoveableObject *
World::getMoveableObject(int index)
{
	return moveableObjects[index];
}

MoveableObject **
World::getMoveableObjects(void)
{
	return moveableObjects;
}

//Sker igen ett worldObject beroende p vilket physicsObject det har
WorldObject *
World::findWorldObject(dGeomID compPhysicsObject)
{
	for(int i = 0; i < numWorldObjects; i++)
	{
		if(worldObjects[i]->getPhysicsObject() == compPhysicsObject)
			return worldObjects[i];
	}

	return 0;
}

MoveableObject *
World::findMoveableObject(dGeomID compPhysicsObject)
{
	for(int i = 0; i < numMoveableObjects; i++)
	{
		if(moveableObjects[i] != 0)
		{
			if(moveableObjects[i]->getPhysicsObject() == compPhysicsObject)
				return moveableObjects[i];
		}
	}

	return 0;
}

void World::stepPhysics(void)
{
	dSpaceCollide(physicsSpace, this, &collisionCallback);
	
	dWorldStep(physicsWorld, 0.01);
	
	dJointGroupEmpty(collisionGroup);
}


void collisionCallback(void *data, dGeomID physicsObject0, dGeomID physicsObject1)
{
	World *world = (World *)data;
	
	//std::cout << "Wohoo - i collisionCallback()!" << std::endl;
	
	WorldObject *worldObject0;
	WorldObject *worldObject1;
	
	worldObject0 = world->findWorldObject(physicsObject0);
	worldObject1 = world->findWorldObject(physicsObject1);
	
	
	dJointID collisionJoint;
	dContact thisContact;
		
	
	if(dCollide(physicsObject0, physicsObject1, 0, &thisContact.geom, sizeof(dContactGeom)))
	{
		
		//Lgga in koll vilka objekt som kolliderat med varandra, tex spelare
		//och boll etc...

		MoveableObject *obj0 = world->findMoveableObject(physicsObject0);
		MoveableObject *obj1 = world->findMoveableObject(physicsObject1);
		
		
		if((obj0 != 0) && (obj1 != 0))
		{
			//printf("Moveables collided!\n");

			if((obj0->asTennisPlayer() != 0) && (obj1->asTennisPlayer() == 0))
			{
				obj0->collision(obj1,thisContact);
			}
			else if((obj1->asTennisPlayer() != 0) && (obj0->asTennisPlayer() == 0))
			{
				obj1->collision(obj0,thisContact);
			}
			else
			{
				obj0->collision(obj1,thisContact);
			}		
		}
		else
		{
			//Standardstuds
			//worldObject0;
			//worldObject1;
			
			thisContact.surface.mode = dContactBounce;
			thisContact.surface.mu = 	worldObject0->getFriction() *
										worldObject1->getFriction();
			//thisContact.surface.mu2 = 1000;
			thisContact.surface.bounce = 	worldObject0->getBounce() *
											worldObject1->getBounce();
			thisContact.surface.bounce_vel = 0;
	
			collisionJoint = dJointCreateContact(world->getPhysicsWorld(), world->getCollisionGroup(), &thisContact);
		
			dJointAttach(collisionJoint, dGeomGetBody(physicsObject0), dGeomGetBody(physicsObject1));
			
		}
		
		Ball *ball;
		if( (ball = worldObject0->asBall()) != 0)
		{
			ball->setLastHit(worldObject1);
			ball->bounceSound();
		}
		if( (ball = worldObject1->asBall()) != 0)
		{
			ball->setLastHit(worldObject0);
			ball->bounceSound();
		}
		
	}
}

void World::setPlayerNr(int pn)
{
	playerNr = pn;
}

int World::getPlayerNr()
{
	return playerNr;
}

void World::updateBallColumns(void)
{
	std::vector<Ball *> *ballList;
	Ball *ball;
	
	for(int i = 0; i < 8; i++)
	{
		if(moveableObjects[i] != 0)
		{
			ballList = moveableObjects[i]->asTennisPlayer()->getBallList();
			
			//std::cout << "Player " << i << std::endl;
			
			for(unsigned int j = 0; j < ballList->size(); j++)
			{
				//std::cout << "ballList size: " << ballList->size() << std::endl;
				
				ball = (*ballList)[j];
				
				dGeomSetPosition(ball->getPhysicsObject(), -50 - i, -50 - (j * 0.3), 0);
			}
		}
	}
}


void
World::setListener()
{
	int i;
	for(i=0;i<numWorldObjects;i++)
	{
		worldObjects[i]->setListener(worldObjects[playerNr]->asTennisPlayer());
	}
	
}
